<?php
/**
 * Plugin Name: WooCommerce Adumo Online Gateway with subscriptions
 * Description: Receive payments through the secure Adumo Online payment gateway and process subscription based payment through Woocomerce Subscriptions addon.
 * Author: Adumo Online
 * Author URI: https://adumoonline.com
 * Version: 2.0.7
 * Requires at least: 5.1
 * Tested up to: 5.9
 * WC tested up to: 6.2.0
 * WC requires at least: 4.5
 * Requires PHP: 7.0+, 7.4+, 8.0+
 */

if (!defined('ABSPATH')) {
    exit;
}

include(dirname(__FILE__).'/php-jwt/jwt.php');

function init_adumoonline_gateway()
{
    class WC_Gateway_AdumoOnline extends WC_Payment_Gateway
    {
        public function __construct()
        {
            $this->id = 'adumoonline';
            $this->icon = WP_PLUGIN_URL.'/woocommerce-adumoonline-gateway-subscriptions/img/Adumo_Online_logo.png';
            $this->has_fields = false;
            $this->method_title = __('Adumo Online Virtual with Subscriptions', 'woocommerce-gateway-adumoonline');
            $this->method_description = __('Accept payments through Adumo Online\'s secure payment gateway and process subscription payments', 'woocommerce-gateway-adumoonline');
            $this->supports = array(
               'products',
               'subscriptions',
               'subscription_cancellation',
               'subscription_suspension',
               'subscription_reactivation',
               'subscription_amount_changes',
               'subscription_date_changes',
               'subscription_payment_method_change',
               'subscription_payment_method_change_customer',
               'subscription_payment_method_change_admin',
               'multiple_subscriptions'
            );

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option('title');

            if ( class_exists( 'WC_Subscriptions_Order' ) ) {
        			add_action('woocommerce_scheduled_subscription_payment_'.$this->id, [ $this, 'scheduled_adumoonline_subscription_payment' ], 10, 2 );
              add_action('woocommerce_scheduled_subscription_payment', array($this, 'scheduled_subscription_payment'), 10);
            }
            //add_action('woocommerce_scheduled_subscription_payment_'.$this->id, array($this, 'scheduled_adumoonline_subscription_payment'));
            add_action('woocommerce_update_options_payment_gateways_'.$this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_receipt_'.$this->id, array($this, 'receipt_page'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce-gateway-adumoonline'),
                    'type' => 'checkbox',
                    'label' => __('Enable Adumo Online Gateway', 'woocommerce-gateway-adumoonline'),
                    'default' => 'yes'
                ),
                'webhook' => array(
                    'title' => __('Activate Webhooks', 'woocommerce-gateway-adumoonline'),
                    'type' => 'checkbox',
                    'label' => __('Activate webhooks for this module. This will need to be configured in the Adumo Online Merchant Portal by adding this webhook endpoint - '.get_site_url().'/wp-json/adumoonline_webhook/v2/process', 'woocommerce-gateway-adumoonline'),
                    'default' => 'no'
                ),
                'jwt_secret' => array(
                    'title' => __('JWT Secret Key', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => __('Your secret key to securly unpack the JWT token, this is configured in the Adumo Online Merchant Portal', 'woocommerce-gateway-adumoonline'),
                    'default' => 'yglTxLCSMm7PEsfaMszAKf2LSRvM2qVW'
                ),
                'title' => array(
                    'title' => __('Title', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                    'default' => __('Adumo Online Credit Card Payment', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'client_secret' => array(
                    'title' => __('Client Secret', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => __('Your client secret is used to process subscription based payments.', 'woocommerce-gateway-adumoonline'),
                    'default' => 'SECRET-0859B53C-AC43-48DE-8367-29CBDD959835'
                ),
                'client_id' => array(
                    'title' => __('Client ID', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => __('Your client ID is used to process subscription based payments.', 'woocommerce-gateway-adumoonline'),
                    'default' => '0859B53C-AC43-48DE-8367-29CBDD959835'
                ),
                'url_oauth' => array(
                    'title' => __('OAuth URL', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get the URL from the Adumo Online developer documentation https://developers.adumoonline.com/. eg: https://apiv3.adumoonline.com/oauth/token',
                    'default' => 'https://staging-apiv3.adumoonline.com/oauth/token'
                ),
                'url_cof' => array(
                    'title' => __('Card on File URL', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get the URL from the Adumo Online developer documentation https://developers.adumoonline.com/. eg: https://apiv3.adumoonline.com/products/payments/v1/card/initiate/cardOnFile',
                    'default' => 'https://staging-apiv3.adumoonline.com/products/payments/v1/card/initiate/cardOnFile'
                ),
                'url_authorise' => array(
                    'title' => __('Authorise URL', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get the URL from the Adumo Online developer documentation https://developers.adumoonline.com/. eg: https://staging-apiv3.adumoonline.com/products/payments/v1/card/authorise',
                    'default' => 'https://staging-apiv3.adumoonline.com/products/payments/v1/card/authorise'
                ),
                'gateway_endpoint' => array(
                    'title' => __('Gateway URL', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get the URL from the Adumo Online developer documentation https://developers.adumoonline.com/. eg: https://apiv3.adumoonline.com/product/payment/v1/initialisevirtual',
                    'default' => 'https://staging-apiv3.adumoonline.com/product/payment/v1/initialisevirtual'
                ),
                'merchant_id' => array(
                    'title' => __('Merchant ID', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get your merchant id by logging into your merchant portal at https://portal.adumoonline.com/login. eg: 9ba5008c-08ee-4286-a349-54af91a621b0',
                    'default' => '9ba5008c-08ee-4286-a349-54af91a621b0'
                ),
                'application_uid' => array(
                    'title' => __('Application UID', 'woocommerce-gateway-adumoonline'),
                    'type' => 'text',
                    'description' => 'You can get your application id by logging into your merchant portal at https://portal.adumoonline.com/login. eg: 1dba1a1b-c4ef-453e-8fa0-dc41d66b629b',
                    'default' => '1dba1a1b-c4ef-453e-8fa0-dc41d66b629b'
                )
            );
        }

        public function process_payment($order_id)
        {
            $order = wc_get_order($order_id);

            return array(
                'result' => 'success',
                'redirect' => $order->get_checkout_payment_url(true),
            );
        }

        public function receipt_page($order)
        {
            echo '<p>'.__('Thank you for your order, please click the button below to pay with Credit Card.', 'woocommerce-gateway-adumoonline').'</p>';
            echo $this->generate_payment_form($order);
        }

        public function generate_payment_form($order_id)
        {
            $order = wc_get_order($order_id);

            // if ( class_exists( 'WC_Subscriptions_Order' ) ) {
            //     $amount = WC_Subscriptions_Order::get_total_initial_payment( $order );
            // } else {
            //     $amount = $order->get_total();
            // }

            $amount = $order->get_total();

            //Build the JWT request token
            $MerchantID = $this->get_option('merchant_id');
            $ApplicationID = $this->get_option('application_uid');
            $amount = $order->get_total();
            $merchantReference = $order_id;
            $SecretKey = $this->get_option('jwt_secret');
            $iss = 'Woocommerce Plugin Subscriptions';
            $authToken = '';

            $payloadArray = array();
            $payloadArray['iss'] = $iss;
            $payloadArray['cuid'] = $MerchantID;
            $payloadArray['auid'] = $ApplicationID;
            $payloadArray['amount'] = $amount;
            $payloadArray['mref'] = $merchantReference;
            $payloadArray['jti'] = base64_encode(random_bytes (32));
            $payloadArray['iat'] = time() - 43200;
            $payloadArray['exp'] = time() + 43200;

            $authToken = JWT::encode($payloadArray, $SecretKey);

            $formFields = array(
                'Mode' => 1,
                'MerchantID' => $MerchantID,
                'ApplicationID' => $ApplicationID,
                'MerchantReference' => $merchantReference,
                'Amount' => $amount,
                'Token' => $authToken,
                'txtCurrencyCode' => $order->get_currency(),
                'RedirectSuccessfulURL' => $this->get_return_url($order),
                'RedirectFailedURL' => $this->get_return_url($order),
                'Discount' => number_format($this->get_order_details($order, 'discount_total'), 2),
                'ShippingCost' => number_format($this->get_order_details($order, 'shipping_total'), 2),
                'Recipient' => $this->get_order_details($order, 'billing_first_name').' '.$this->get_order_details($order, 'billing_last_name'),
                'ShippingAddress1' => $this->get_order_details($order, 'shipping_address_1'),
                'ShippingAddress2' => $this->get_order_details($order, 'shipping_address_2'),
                'ShippingAddress3' => $this->get_order_details($order, 'shipping_city'),
                'ShippingAddress4' => $this->get_order_details($order, 'shipping_postcode'),
                'ShippingAddress5' => $this->get_order_details($order, 'shipping_country'),
            );

            $counter = 1;
            foreach ($order->get_items() as $product) {
                /** @var WC_Order_Item_Product $product */

                $formFields["Qty$counter"] = $product->get_quantity();
                $formFields["ItemRef$counter"] = $product->get_id();
                $formFields["ItemDescr$counter"] = $product->get_name();
                $formFields["ItemAmount$counter"] = number_format($product->get_total(), 2);

                $counter++;
            }

            $form = '<form action="'.esc_url($this->get_option('gateway_endpoint')).'" method="post" id="adumoonline_payment_form">';

            foreach ($formFields as $key => $value) {
                $form .= '<input type="hidden" name="'.esc_attr($key).'" value="'.esc_attr($value).'" />';
            }

            $form .= '
				<input type="submit" class="button-alt" id="submit_adumoonline_payment_form" value="'.__('Pay with Credit Card', 'woocommerce-gateway-adumoonline').'" /> <a class="button cancel" href="'.$order->get_cancel_order_url().'">'.__('Cancel order &amp; restore cart', 'woocommerce-gateway-adumoonline').'</a>
				<script type="text/javascript">
					jQuery(function(){
						jQuery("body").block(
							{
								message: "<img src=\"'.$this->icon.'\" /><br />'.__('Thank you for your order. You will now be redirected to Adumo Online\'s secure payment page to make payment.', 'woocommerce-gateway-adumoonline').'",
								overlayCSS:
								{
									background: "#fff",
									opacity: 0.6
								},
								css: {
									padding:        20,
									textAlign:      "center",
									color:          "#555",
									border:         "3px solid #aaa",
									backgroundColor:"#fff",
									cursor:         "wait"
								}
							});
						jQuery( "#submit_adumoonline_payment_form" ).click();
					});
				</script>
			</form>';

            return $form;
        }

        public static function check_status_response()
        {

            //Get admin settings and validate the JWT token
            $gateway_settings = WC()->payment_gateways->payment_gateways()['adumoonline'];
            $jwtSecret = $gateway_settings->settings['jwt_secret'];

            //Check if JWT token is in the response and that a secret key is set
            if(isset($_POST['_RESPONSE_TOKEN']) && !empty($_POST['_RESPONSE_TOKEN']) && isset($jwtSecret)) {

              $jwtToken = $_POST['_RESPONSE_TOKEN'];
              $type = 'normal';

              // if(isset($_POST['VARIABLE1']) && $_POST['VARIABLE1']) {
              //     $key = $_POST['VARIABLE1'];
              // } elseif(isset($_POST['Variable1']) && $_POST['Variable1']) {
              //     $key = $_POST['Variable1'];
              // } else {
              //     $key = null;
              // }

              if(isset($_GET['key'])) {
                  $key = $_GET['key'];
              } else {
                  $key = null;
              }

              error_log("Adumo Response-----------------------------------".print_r($_POST,1));

              process_the_order($key,$jwtToken,$jwtSecret,$type);

            } else {
              //No token in the response
              if(isset($_POST['_MERCHANTREFERENCE'])) {
                  $order_id = (int) (isset($_POST['_MERCHANTREFERENCE']) ? $_POST['_MERCHANTREFERENCE'] : 0);
                  $order = wc_get_order($order_id);
                  if(isset($_POST['_ERROR_MESSAGE']) && !empty($_POST['_ERROR_MESSAGE'])) {
                      $message = $_POST['_ERROR_MESSAGE'];
                      error_log("Adumo Online Payment Failed: ".print_r($message,1));
                      $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
                  } else {
                      error_log('JWT Token not found in the response from AdumoOnline!');
                      $message = 'JWT Token not found in the response from AdumoOnline!';
                      $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
                  }
              }

            }

        }

        private function get_order_details($order, $prop)
        {
            switch ($prop) {
                case 'order_total':
                    $getter = array($order, 'get_total');
                    break;
                default:
                    $getter = array($order, 'get_'.$prop);
                    break;
            }

            return is_callable($getter) ? call_user_func($getter) : $order->{$prop};
        }

        public function scheduled_subscription_payment($subscription_id)
            {
                //error_log("scheduled_subscription_payment-----------------------------------".print_r($subscription_id,1));
            }

        public function scheduled_adumoonline_subscription_payment($amount_to_charge, $renewal_order) {
            error_log('Function called to process recurring payment');

            $renewal_order_id = $renewal_order->get_id();

            if ( wcs_order_contains_renewal( $renewal_order_id ) ) {
                $subscriptions = wcs_get_subscriptions_for_order( $renewal_order_id, array( 'order_type' => 'any' ) );
                if(!empty($subscriptions)) {
                    foreach( $subscriptions as $subscription_id => $subscription_obj )
                        {
                           $parent_id = $subscription_obj->get_parent_id();
                        }
                    $parent_order = wc_get_order($parent_id);
                    $cardToken = $parent_order->get_meta('card_token');
                    //$cardToken = get_post_meta( $parent_id, 'card_token', true );
                    error_log("Card Token Retrtieved-----------------------------------".print_r($cardToken,1));
                    $renewal_order->add_order_note(__(sprintf('Adumo Online Card Token Retrieved. Card Token: %s', $cardToken), 'woocommerce-gateway-adumoonline'));
                }
            }

            if(!empty($cardToken)) {

                $clientID = $this->get_option('client_id');
                $clientSecret = $this->get_option('client_secret');
                $urlOauth = $this->get_option('url_oauth');
                $urlCardOnFile = $this->get_option('url_cof');
                $urlAuthorise = $this->get_option('url_authorise');
                $urlSettlement = '';

                $cofValue = $renewal_order->get_total();
                $applicationUid = $this->get_option('application_uid');
                $merchantUid = $this->get_option('merchant_id');
                $budgetPeriod = 0;
                $description = 'WC Subscription Renewal';
                $merchantReference = $renewal_order_id;
                $uci = '70.32.90.61';

                //OAuth request
                $data = array(
                    'grant_type' => 'client_credentials',
                    'client_id' => $clientID,
                    'client_secret' => $clientSecret
                );
                $curl = curl_init();

                curl_setopt_array($curl, array(
                  CURLOPT_URL => $urlOauth,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => '',
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 0,
                  CURLOPT_FOLLOWLOCATION => true,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => 'POST',
                  CURLOPT_POSTFIELDS => $data,
                ));

                $response = curl_exec($curl);
                curl_close($curl);
                $response = json_decode($response);

                error_log("OAuth Response-----------------------------------".print_r($response,1));

                if(!empty($response->access_token)) {

                    //Card on File Request
                    $cofRequest = array(
                      "applicationUid" => $applicationUid,
                      "merchantUid" => $merchantUid,
                      "budgetPeriod" => $budgetPeriod,
                      "description" => $description,
                      "merchantReference" => $merchantReference,
                      "value" => $cofValue,
                      "token" => $cardToken,
                      "ipAddress" => "172.23.36.21"
                    );

                    $cofRequest = json_encode($cofRequest);

                    $curl = curl_init();

                    curl_setopt_array($curl, array(
                      CURLOPT_URL => $urlCardOnFile,
                      CURLOPT_RETURNTRANSFER => true,
                      CURLOPT_ENCODING => '',
                      CURLOPT_MAXREDIRS => 10,
                      CURLOPT_TIMEOUT => 0,
                      CURLOPT_FOLLOWLOCATION => true,
                      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                      CURLOPT_CUSTOMREQUEST => 'POST',
                      CURLOPT_POSTFIELDS => $cofRequest,
                      CURLOPT_HTTPHEADER => array(
                        'Authorization: Bearer '.$response->access_token.'',
                        'Content-Type: application/json'
                      ),
                    ));

                    $cofResponse = curl_exec($curl);
                    curl_close($curl);
                    $cofResponse = json_decode($cofResponse);

                    error_log("Card on File Response-----------------------------------".print_r($cofResponse,1));

                    if(!empty($cofResponse->transactionId)) {

                        //Authorise request
                        $authorise_request = array(
                            'transactionId' => $cofResponse->transactionId,
                            'amount' => $cofValue,
                            'cvv' => ''
                        );

                        $authorise_request = json_encode($authorise_request);

                        $curl = curl_init();

                        curl_setopt_array($curl, array(
                          CURLOPT_URL => $urlAuthorise,
                          CURLOPT_RETURNTRANSFER => true,
                          CURLOPT_ENCODING => '',
                          CURLOPT_MAXREDIRS => 10,
                          CURLOPT_TIMEOUT => 0,
                          CURLOPT_FOLLOWLOCATION => true,
                          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                          CURLOPT_CUSTOMREQUEST => 'POST',
                          CURLOPT_POSTFIELDS => $authorise_request,
                          CURLOPT_HTTPHEADER => array(
                            'Authorization: Bearer '.$response->access_token.'',
                            'Content-Type: application/json'
                          ),
                        ));

                        $authResponse = curl_exec($curl);
                        curl_close($curl);
                        $authResponse = json_decode($authResponse);

                        error_log("Authorise Response-----------------------------------".print_r($authResponse,1));

                        if(isset($authResponse->statusCode) && !empty($authResponse->statusCode) && $authResponse->statusCode == 200) {
                            $renewal_order->add_order_note(__(sprintf('Adumo Online subscription payment completed. Transaction Id: %s', $cofResponse->transactionId), 'woocommerce-gateway-adumoonline'));
                            $renewal_order->add_order_note(__(sprintf('Adumo Online subscription payment completed. Card Token: %s', $cardToken), 'woocommerce-gateway-adumoonline'));
                            $renewal_order->add_order_note(__(sprintf('Adumo Online subscription payment completed. Status Code: %s', $authResponse->statusCode), 'woocommerce-gateway-adumoonline'));
                            $renewal_order->add_order_note(__(sprintf('Adumo Online subscription payment authorised. Bank Message: %s', $authResponse->statusMessage), 'woocommerce-gateway-adumoonline'));
                            $renewal_order->payment_complete( $cofResponse->transactionId );
                        } else {
                            $renewal_order->add_order_note(__(sprintf('Adumo Online subscription payment failed. Status Code: %s', $authResponse->errorCode), 'woocommerce-gateway-adumoonline'));
                            $renewal_order->add_order_note(__(sprintf('Error Message: %s', $authResponse->message), 'woocommerce-gateway-adumoonline'));
                        }

                    } else {

                        error_log("Error calling Card on File service");
                        $renewal_order->add_order_note(__(sprintf('Error calling Card on File service: %s', $cofResponse->message), 'woocommerce-gateway-adumoonline'));

                    }

                } else {

                    error_log("Error calling OAuth service");
                    $renewal_order->add_order_note(__(sprintf('Error calling OAuth service: %s', $response->message), 'woocommerce-gateway-adumoonline'));

                }

            } else {

                error_log("Card Token not found");
                $renewal_order->add_order_note(__(sprintf('Error - Card Token not found'), 'woocommerce-gateway-adumoonline'));

            }

            //Settlement request
            // $settle_request = array(
            //     'transactionId' => 'client_credentials',
            //     'amount' => $cofValue
            // );
            //
            // $settle_request = json_encode($settle_request);
            //
            // $curl = curl_init();
            //
            // curl_setopt_array($curl, array(
            //   CURLOPT_URL => $urlSettlement,
            //   CURLOPT_RETURNTRANSFER => true,
            //   CURLOPT_ENCODING => '',
            //   CURLOPT_MAXREDIRS => 10,
            //   CURLOPT_TIMEOUT => 0,
            //   CURLOPT_FOLLOWLOCATION => true,
            //   CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            //   CURLOPT_CUSTOMREQUEST => 'POST',
            //   CURLOPT_POSTFIELDS => $settle_request,
            //   CURLOPT_HTTPHEADER => array(
            //     'Authorization: Bearer '.$response->access_token.'',
            //     'Content-Type: application/json'
            //   ),
            // ));
            //
            // $settleResponse = curl_exec($curl);
            // curl_close($curl);
            // $settleResponse = json_decode($settleResponse);
            //
            // error_log("Settlement Response-----------------------------------".print_r($settleResponse,1));

        }

    }

    add_filter('woocommerce_payment_gateways', 'add_adumoonline_gateway_class');
}

function process_the_order($key,$jwtToken,$jwtSecret,$type) {

    $data = JWT::decode($jwtToken, $jwtSecret, array('HS256'));
    error_log("JWT Response-----------------------------------".print_r($data,1));
    $mref = $data->mref;
    $result = $data->result;
    $transactionIndex = $data->transactionIndex;
    $amount = $data->amount;
    $valid = $data->verify;
    $cardToken = 'N/A';
    if(isset($data->tkn)) {
        $cardToken = $data->tkn;
    }

    $order_id = (int) (isset($mref) ? $mref : 0);

    if (0 === $order_id) {
        return;
    }

    $order = wc_get_order($order_id);

    if ($order->has_status(array('processing', 'complete'))) {
        return;
    }

    $result = isset($result) ? (int) $result : false;

    if (false === $result) {
        return;
    }

    if ($key !== $order->get_order_key() && $type == 'normal') {
        return wp_redirect($order->get_cancel_order_url());
    }

    if ($result >= 0) {
        $transactionIndex = isset($transactionIndex) ? $transactionIndex : 'N/A';
        $order->add_order_note(__(sprintf('Adumo Online payment completed. Transaction Id: %s', $transactionIndex), 'woocommerce-gateway-adumoonline'));
        if($cardToken != 'N/A') {
            $subscriptions = wcs_get_subscriptions_for_order( $order_id, array( 'order_type' => 'any' ) );
            if(!empty($subscriptions)) {
                foreach( $subscriptions as $subscription_id => $subscription_obj )
                    {
                       $parent_id = $subscription_obj->get_parent_id(); // This is subscription parent id
                    }
                update_post_meta( $parent_id, 'card_token', $cardToken);
                error_log("Card Token Saved-----------------------------------".print_r($cardToken,1));
                $order->add_order_note(__(sprintf('Adumo Online Card Token Saved. Card Token: %s', $cardToken), 'woocommerce-gateway-adumoonline'));
            }

        }
        $order->payment_complete($transactionIndex);

        //Validate the token
        if ($valid) {
            error_log("JWT Token Verified");
            $order->add_order_note(__(sprintf('JWT Token Verified'), 'woocommerce-gateway-adumoonline'));
        } else {
            error_log('JWT Token does not have valid signature!');
            $order->add_order_note(__(sprintf('JWT Token does not have valid signature!'), 'woocommerce-gateway-adumoonline'));
            $order->update_status( 'on-hold' );
        }

        //Validate the amount
        if ( $order->get_total() == $amount ) {
            error_log("Amount validation passed");
            $order->add_order_note(__(sprintf('Amount validation passed'), 'woocommerce-gateway-adumoonline'));
        } else {
            error_log('Amount validation failed - order amount and gateway amount are different!');
            $order->add_order_note(__(sprintf('Amount validation failed - order amount and gateway amount are different!'), 'woocommerce-gateway-adumoonline'));
            $order->update_status( 'on-hold' );
        }

    } else {
        $message = isset($_POST['_BANK_ERROR_MESSAGE']) ? $_POST['_BANK_ERROR_MESSAGE'] : (isset($_POST['_ERROR_DETAIL']) ? $_POST['_ERROR_DETAIL'] : (isset($_POST['_ERROR_MESSAGE']) ? $_POST['_ERROR_MESSAGE'] : 'Unknown Error'));
        $order->update_status('failed', __(sprintf('Adumo Online payment failed: %s', $message), 'woocommerce-gateway-adumoonline'));
    }
}

function add_adumoonline_gateway_class($methods)
{
    $methods[] = 'WC_Gateway_AdumoOnline';

    return $methods;
}

add_action('plugins_loaded', 'init_adumoonline_gateway', 0);
add_action('init', array('WC_Gateway_AdumoOnline', 'check_status_response'), 255);

/**
 * Checks if the WooCommerce plugin is activated
 *
 * If the WooCommerce plugin is not active, then don't allow the
 * activation of this plugin.
 *
 * @since 1.0.0
 */
function adumoonline_activate() {
  if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
    include_once( ABSPATH . '/wp-admin/includes/plugin.php' );
  }
  if ( current_user_can( 'activate_plugins' ) && ( ! class_exists( 'WooCommerce' ) || ! class_exists( 'WC_Subscriptions' ) ) ) {
    // Deactivate the plugin.
    deactivate_plugins( plugin_basename( __FILE__ ) );
    // Throw an error in the WordPress admin console.
    $error_message = '<p style="font-family:-apple-system,BlinkMacSystemFont,\'Segoe UI\',Roboto,Oxygen-Sans,Ubuntu,Cantarell,\'Helvetica Neue\',sans-serif;font-size: 13px;line-height: 1.5;color:#444;">' . esc_html__( 'This plugin requires ', 'adumoonline' ) . '<a href="' . esc_url( 'https://woocommerce.com/' ) . '">WooCommerce</a>' . esc_html__( ' and ', 'adumoonline' ) . '<a href="' . esc_url( 'https://woocommerce.com/products/woocommerce-subscriptions/' ) . '">WooCommerce Subscriptions</a>' . esc_html__( ' plugins to be active.', 'adumoonline' ) . '</p>';
    die( $error_message ); // WPCS: XSS ok.
  }
}
register_activation_hook( __FILE__, 'adumoonline_activate' );


//Setup webhook endpoint
function adumoonline_webhook_v2(WP_REST_Request $request) {
    $data = $request->get_params();
    //$order = wc_get_order($data['merchantReference']);
    error_log("Webhook Response-----------------------------------".print_r($data,1));
    //Get admin settings and validate the JWT token
     $gateway_settings = WC()->payment_gateways->payment_gateways()['adumoonline'];
     $jwtSecret = $gateway_settings->settings['jwt_secret'];
     $jwtToken = $data['token'];

     $token = JWT::decode($jwtToken, $jwtSecret, array('HS256'));

     // $mref = $data->mref;
     // $result = $data->result;
     // $transactionIndex = $data->transactionIndex;
     // $amount = $data->amount;
     // $valid = $data->verify;

    // $type = 'normal';
    // process_the_order($key,$jwtToken,$jwtSecret,$type);

    error_log("Webhook received");
    error_log("Webhook JWT Response-----------------------------------".print_r($token,1));

    // error_log("Unpack JWT");
    // $claims = $token->getClaims();
    // error_log( 'VALID: '. $valid );
    // foreach ($claims as $key => $claim) {
  	// 	error_log( $key . ': '. $token->getClaim($key) );
  	// }

    // if ($order->has_status(array('processing', 'complete'))) {
    //     return TRUE;
    // } else {
    //     if ($data['status'] == 'SETTLED' || $data['status'] == 'APPROVED') {
    //         $transactionIndex = isset($data['transactionId']) ? $data['transactionId'] : 'N/A';
    //         $order->add_order_note(__(sprintf('Adumo Online payment completed via webhook. Transaction Id: %s', $transactionIndex), 'woocommerce-gateway-adumoonline'));
    //         $order->payment_complete($transactionIndex);
    //     } else {
    //         $message = isset($data['message']) ? $data['message'] : 'Unknown Error';
    //         $order->update_status('failed', __(sprintf('Adumo Online payment failed via webhook: %s', $message), 'woocommerce-gateway-adumoonline'));
    //     }
    // }

    return TRUE;
}
//Register the webhook endpoint v1
function register_adumoonline_webhooks_routes_v2() {
    register_rest_route( 'adumoonline_webhook/v2', '/process', array(
        'methods'  => 'POST', //WP_REST_Server::READABLE,
        'callback' => 'adumoonline_webhook_v2',
        'permission_callback' => '__return_true'
    ) );
}
add_action( 'rest_api_init', 'register_adumoonline_webhooks_routes_v2' );
